@@ -1,35 +1,53 @@
 from datetime import datetime
 import socket
 import struct
-from getent.constants import *
-from getent.libc import *
+import sys
+from getent.constants import * # NOQA
+from getent.libc import * # NOQA
 from getent import headers
 
 
+if sys.version_info[0] < 3:
+    def convert23(value):
+        return value
+
+else:
+    def convert23(value): # NOQA
+        return value.decode('utf-8') if isinstance(value, bytes) else value
+
+
 class StructMap(object):
+
     def __init__(self, p):
         self.p = p
+
         for attr in dir(self.p.contents):
+
             if attr.startswith('_'):
                 continue
+
             elif not hasattr(self, attr):
-                setattr(self, attr, getattr(self.p.contents, attr))
+                value = getattr(self.p.contents, attr)
+                setattr(self, attr, convert23(value))
 
     def __dict__(self):
         return dict(iter(self))
 
     def __iter__(self):
         for attr in dir(self.p.contents):
+
             if attr.startswith('_'):
                 continue
+
             else:
                 yield (attr, getattr(self, attr))
 
     def _map(self, attr):
         i = 0
         obj = getattr(self.p.contents, attr)
+
         while obj[i]:
-            yield obj[i]
+            yield convert23(obj[i])
             i += 1
 
 
@@ -42,8 +60,8 @@ def _resolve(addrtype, addr):
         return socket.inet_ntop(addrtype, packed)
     elif addrtype == AF_INET6:
         p = cast(addr, POINTER(headers.InAddr6Struct))
-        packed = ''.join(map(lambda bit: struct.pack('<L', bit),
-            p.contents.in6_u.u6_addr32))
+        packed = ''.join([struct.pack('<L', bit)
+                         for bit in p.contents.in6_u.u6_addr32])
         return socket.inet_ntop(addrtype, packed)
 
 
@@ -51,8 +69,8 @@ class Host(StructMap):
     def __init__(self, p):
         super(Host, self).__init__(p)
         self.aliases = list(self._map('aliases'))
-        self.addresses = map(lambda addr: _resolve(self.addrtype, addr), 
-            self._map('addr_list'))
+        self.addresses = [_resolve(self.addrtype, addr)
+                          for addr in self._map('addr_list')]
 
 
 class Proto(StructMap):
@@ -103,7 +121,20 @@ class Shadow(StructMap):
 
 
 def alias(search=None):
-    # Iterate over all alias entries
+    '''
+    Perform a (mail) alias lookup.
+
+    To iterate over all alias entries::
+
+        >>> for item in alias():
+        ...
+
+    To lookup a single alias::
+
+        >>> mail = alias('postmaster')
+        >>> print mail.members
+        root
+    '''
     if search is None:
         setaliasent()
         p = True
@@ -115,8 +146,24 @@ def alias(search=None):
         endaliasent()
         return r
 
+
 def host(search=None):
-    # Iterate over all host entries
+    '''
+    Perform a host(s) lookup.
+
+    To iterate over all host entries::
+
+        >>> for item in host():
+        ...
+
+    To lookup a single host by name::
+
+        >>> import socket
+        >>> server = host('localhost')
+        >>> server.addrtype in [socket.AF_INET, socket.AF_INET6]
+        True
+
+    '''
     if search is None:
         sethostent()
         p = True
@@ -156,7 +203,22 @@ def host(search=None):
         if bool(host):
             return Host(host)
 
+
 def proto(search=None):
+    '''
+    Perform a protocol lookup.
+
+    To lookup all protocols::
+
+        >>> for item in proto():
+        ...
+
+    To lookup a single protocol number::
+
+        >>> tcp = proto('tcp')
+        >>> print tcp.proto
+        6
+    '''
     if search is None:
         setprotoent()
         prt = True
@@ -171,14 +233,32 @@ def proto(search=None):
     else:
         search = str(search)
         if search.isdigit():
-            prt = getprotobynumber(uid_t(long(search)))
+            prt = getprotobynumber(uid_t(int(search)))
         else:
             prt = getprotobyname(c_char_p(search))
 
         if bool(prt):
             return Proto(prt)
 
+
 def rpc(search=None):
+    '''
+    Perform a remote procedure call lookup.
+
+    To lookup all rpc services::
+
+        >>> for item in rpc():
+        ...
+
+    To lookup one rpc service by name::
+
+        >>> nfs = rpc('nfs')
+        >>> print nfs.number
+        100003
+        >>> print nfs.aliases
+        ['portmap', 'sunrpc']
+
+    '''
     if search is None:
         setrpcent()
         rpc = True
@@ -193,13 +273,14 @@ def rpc(search=None):
     else:
         search = str(search)
         if search.isdigit():
-            rpc = getrpcbynumber(uid_t(long(search)))
+            rpc = getrpcbynumber(uid_t(int(search)))
         else:
             rpc = getrpcbyname(c_char_p(search))
 
         if bool(rpc):
             return RPC(rpc)
 
+
 def service(search=None, proto=None):
     '''
     Perform a service lookup.
@@ -246,13 +327,14 @@ def service(search=None, proto=None):
         if not proto in ['tcp', 'udp']:
             raise ValueError('Unsupported protocol "%s"' % (str(proto),))
         if search.isdigit():
-            srv = getservbyport(uid_t(long(search)), c_char_p(proto))
+            srv = getservbyport(uid_t(int(search)), c_char_p(proto))
         else:
             srv = getservbyname(c_char_p(search), c_char_p(proto))
 
         if bool(srv):
             return Service(srv)
 
+
 def network(search=None):
     '''
     Perform a network lookup.
@@ -283,6 +365,7 @@ def network(search=None):
         if bool(net):
             return Network(net)
 
+
 def group(search=None):
     '''
     Perform a group lookup.
@@ -320,13 +403,14 @@ def group(search=None):
     else:
         search = str(search)
         if search.isdigit():
-            grp = getgrgid(gid_t(long(search)))
+            grp = getgrgid(gid_t(int(search)))
         else:
             grp = getgrnam(c_char_p(search))
 
         if bool(grp):
             return Group(grp)
 
+
 def passwd(search=None):
     '''
     Perform a passwd lookup.
@@ -364,13 +448,14 @@ def passwd(search=None):
     else:
         search = str(search)
         if search.isdigit():
-            pwd = getpwuid(uid_t(long(search)))
+            pwd = getpwuid(uid_t(int(search)))
         else:
             pwd = getpwnam(c_char_p(search))
 
         if bool(pwd):
             return Passwd(pwd)
 
+
 def shadow(search=None):
     '''
     Perform a shadow lookup.
@@ -382,13 +467,15 @@ def shadow(search=None):
 
     To lookup one user by name::
 
-        >>> root = shadow('root') 
+        >>> root = shadow('root')
         >>> print root.warn # doctest: +SKIP
         99999
 
     '''
     # Iterate over all shadow entries
     if search is None:
+        if setspent is None:
+            raise NotImplementedError('Not available on %s' % (sys.platform,))
         setspent()
         spe = True
         res = []
@@ -404,32 +491,36 @@ def shadow(search=None):
         if bool(spe):
             return Shadow(spe)
 
+
 if __name__ == '__main__':
-    print dict(host('127.0.0.1'))
-    print dict(host('localhost'))
+    print(dict(host('127.0.0.1')))
+    print(dict(host('localhost')))
 
     for h in host():
-        print dict(h)
+        print(dict(h))
 
-    print dict(network('link-local') or {})
+    print(dict(network('link-local') or {}))
 
     for p in proto():
-        print p.name, p.aliases
+        print(p.name, p.aliases)
 
     for r in rpc():
-        print r.name, r.aliases
+        print(r.name, r.aliases)
 
     for s in service():
-        print s.name, s.port, s.proto
+        print(s.name, s.port, s.proto)
 
     for n in network():
-        print n.name, n.aliases
+        print(n.name, n.aliases)
 
     for g in group():
-        print g.name, g.members, dict(g)
+        print(g.name, g.members, dict(g))
 
     for p in passwd():
-        print p.name, p.shell, p.dir, dict(p)
+        print(p.name, p.shell, p.dir, dict(p))
 
-    for s in shadow():
-        print s.name, s.change, s.expire, dict(s)
+    try:
+        for s in shadow():
+            print(s.name, s.change, s.expire, dict(s))
+    except NotImplementedError as e:
+        print('shadow() failed:', e)
